package com.sleepycat.je.checksum;

import java.nio.ByteBuffer;

import com.sleepycat.je.log.*;
import com.sleepycat.je.mock.*;



aspect Checksum advises 
		LogManagerInitializationJP, 
		ValidateHeaderJP,
		ValidateBodyJP,
		LogManagerStaticInitJP,
		FileReaderInitializationJP,
		FileReaderValidateChecksumJP,
		FileReaderDeterminIsCollectDataJP,
		FileReaderReadHeaderJP,
		FileReaderStartChecksumJP {
	
	after(LogManagerInitializationJP l) {
		/* See if we're configured to do a checksum when reading in objects. */
		l._this.doChecksumOnRead = true;
	}
	
	private ChecksumValidator validator;
	private long storedChecksum;
	
	after(ValidateHeaderJP l) throws DatabaseException {
		System.out.println(thisJoinPoint);
		validator = null;
		storedChecksum = LogUtils.getUnsignedInt(l.entryBuffer);
		if (l._this.doChecksumOnRead) {
			validator = new ChecksumValidator();
			validator.update(l._this.envImpl, l.entryBuffer, LogManager.HEADER_CONTENT_BYTES,
					false);
		}

	}
	after(ValidateBodyJP l) throws DatabaseException {
		System.out.println(thisJoinPoint);
		if (l._this.doChecksumOnRead) {
			 /* Check the checksum first. */
			 validator.update(l._this.envImpl, l.entryBuffer, l.itemSize, false);
			 validator.validate(l._this.envImpl, storedChecksum, l.lsn);
		 }
	}
	
	public boolean LogManager.doChecksumOnRead;
	public boolean LogManager.getChecksumOnRead() {
		return doChecksumOnRead;
	}
	
	static final int LogManager.CHECKSUM_BYTES = 4; // size of checksum field

	static final int LogManager.HEADER_CHECKSUM_OFFSET = 0; // size of checksum

	// field

	after(LogManagerStaticInitJP j) {
		LogManager.HEADER_BYTES += LogManager.CHECKSUM_BYTES; // size of entry
		// header

		LogManager.PREV_BYTES = 4; // size of previous field

		LogManager.HEADER_CONTENT_BYTES = LogManager.HEADER_BYTES
				- LogManager.CHECKSUM_BYTES;

		LogManager.HEADER_ENTRY_TYPE_OFFSET += 4;

		LogManager.HEADER_VERSION_OFFSET += 4;

		LogManager.HEADER_PREV_OFFSET += 4;

		LogManager.HEADER_SIZE_OFFSET += 4;
	}
	
	// //////////////////////////////////////////////////////////
	
	before(FileReaderInitializationJP j) throws DatabaseException {
		j._this.doValidateChecksum = j.env.getLogManager().getChecksumOnRead();
	}
	after(FileReaderInitializationJP j) {
		if (j._this.doValidateChecksum) {
			j._this.cksumValidator = new ChecksumValidator();
		}
		j._this.anticipateChecksumErrors = false;
	}
	
	boolean around(FileReaderDeterminIsCollectDataJP j) {
		return proceed(j) || j._this.doValidateChecksum && j._this.alwaysValidateChecksum;
	}
	
	before(FileReaderStartChecksumJP j) throws DatabaseException {
		boolean doValidate = j._this.doValidateChecksum
			&& (j.isTargetEntry || j._this.alwaysValidateChecksum);
		if (doValidate) {
			j._this.startChecksum(j.dataBuffer);
		}
	}
	
	before(FileReaderValidateChecksumJP j) throws DatabaseException {
		boolean doValidate = j._this.doValidateChecksum
			&& (j.isTargetEntry || j._this.alwaysValidateChecksum);
		if (doValidate) {
			j._this.validateChecksum(j.dataBuffer);
		}
	}

	before(FileReaderReadHeaderJP j){
		j._this.currentEntryChecksum = LogUtils.getUnsignedInt(j.dataBuffer);
	}
	
	
	public ChecksumValidator FileReader.cksumValidator;
	public boolean FileReader.doValidateChecksum; // Validate checksums
	public boolean FileReader.alwaysValidateChecksum; // Validate for all
	public boolean FileReader.anticipateChecksumErrors;
	public void FileReader.startChecksum(ByteBuffer dataBuffer)
			throws DatabaseException {
		cksumValidator.reset();
		int entryStart = threadSafeBufferPosition(dataBuffer);
		System.out.println(dataBuffer);
		dataBuffer.reset();
		cksumValidator.update(env, dataBuffer, LogManager.HEADER_CONTENT_BYTES,
				anticipateChecksumErrors);
		threadSafeBufferPosition(dataBuffer, entryStart);
	}
	public long FileReader.currentEntryChecksum;
	public void FileReader.validateChecksum(ByteBuffer entryBuffer)
			throws DatabaseException {

		cksumValidator.update(env, entryBuffer, currentEntrySize,
				anticipateChecksumErrors);
// cksumValidator.validate(env, currentEntryChecksum, readBufferFileNum,
// currentEntryOffset, anticipateChecksumErrors);
	}
}